@@ -6,11 +6,12 @@ import (
 
 	"github.com/livebud/bud/internal/fscache"
 	"github.com/livebud/bud/internal/is"
+	"github.com/livebud/bud/package/log/testlog"
 )
 
 func TestFakeDir(t *testing.T) {
 	is := is.New(t)
-	fmap := fscache.New()
+	fmap := fscache.New(testlog.New())
 	fmap.Set("index.svelte", &fscache.File{
 		Name: "index.svelte",
 		Data: []byte(`<h1>hello</h1>`),
@@ -23,7 +24,7 @@ func TestFakeDir(t *testing.T) {
 
 func TestFile(t *testing.T) {
 	is := is.New(t)
-	fmap := fscache.New()
+	fmap := fscache.New(testlog.New())
 	fmap.Set("index.svelte", &fscache.File{
 		Name: "index.svelte",
 		Data: []byte(`<h1>hello</h1>`),
