@@ -32,12 +32,12 @@ func writeFiles(dir string, files map[string]string) error {
 	return eg.Wait()
 }
 
-func getEvent(event <-chan []string) ([]string, error) {
+func getEvent(event <-chan []watcher.Event) ([]watcher.Event, error) {
 	select {
-	case paths := <-event:
-		return paths, nil
+	case events := <-event:
+		return events, nil
 	case <-time.After(1 * time.Second):
-		return nil, errors.New("timed out while waiting for watcher")
+		return nil, errors.New("timed out while waiting for watcher events")
 	}
 }
 
@@ -49,14 +49,14 @@ func TestChange(t *testing.T) {
 	})
 	is.NoErr(err)
 	ctx := context.Background()
-	event := make(chan []string)
+	eventCh := make(chan []watcher.Event)
 	ctx, cancel := context.WithCancel(context.Background())
 	defer cancel()
 	eg := new(errgroup.Group)
 	eg.Go(func() error {
-		return watcher.Watch(ctx, dir, func(paths []string) error {
+		return watcher.Watch(ctx, dir, func(events []watcher.Event) error {
 			select {
-			case event <- paths:
+			case eventCh <- events:
 			case <-ctx.Done():
 			}
 			return nil
@@ -65,10 +65,11 @@ func TestChange(t *testing.T) {
 	time.Sleep(waitForEvents)
 	err = os.WriteFile(filepath.Join(dir, "a.txt"), []byte("b"), 0644)
 	is.NoErr(err)
-	paths, err := getEvent(event)
+	events, err := getEvent(eventCh)
 	is.NoErr(err)
-	is.Equal(len(paths), 1)
-	is.Equal(paths[0], filepath.Join(dir, "a.txt"))
+	is.Equal(len(events), 1)
+	is.Equal(events[0].Path, filepath.Join(dir, "a.txt"))
+	is.Equal(events[0].Op, watcher.OpUpdate)
 	cancel()
 	is.NoErr(eg.Wait())
 }
@@ -81,14 +82,14 @@ func TestDelete(t *testing.T) {
 	})
 	is.NoErr(err)
 	ctx := context.Background()
-	event := make(chan []string)
+	eventCh := make(chan []watcher.Event)
 	ctx, cancel := context.WithCancel(context.Background())
 	defer cancel()
 	eg := new(errgroup.Group)
 	eg.Go(func() error {
-		return watcher.Watch(ctx, dir, func(paths []string) error {
+		return watcher.Watch(ctx, dir, func(events []watcher.Event) error {
 			select {
-			case event <- paths:
+			case eventCh <- events:
 			case <-ctx.Done():
 			}
 			return nil
@@ -97,10 +98,11 @@ func TestDelete(t *testing.T) {
 	time.Sleep(waitForEvents)
 	err = os.RemoveAll(filepath.Join(dir, "a.txt"))
 	is.NoErr(err)
-	paths, err := getEvent(event)
+	events, err := getEvent(eventCh)
 	is.NoErr(err)
-	is.Equal(len(paths), 1)
-	is.Equal(paths[0], filepath.Join(dir, "a.txt"))
+	is.Equal(len(events), 1)
+	is.Equal(events[0].Path, filepath.Join(dir, "a.txt"))
+	is.Equal(events[0].Op, watcher.OpDelete)
 	cancel()
 	is.NoErr(eg.Wait())
 }
@@ -109,14 +111,14 @@ func TestCreate(t *testing.T) {
 	is := is.New(t)
 	dir := t.TempDir()
 	ctx := context.Background()
-	event := make(chan []string)
+	eventCh := make(chan []watcher.Event)
 	ctx, cancel := context.WithCancel(context.Background())
 	defer cancel()
 	eg := new(errgroup.Group)
 	eg.Go(func() error {
-		return watcher.Watch(ctx, dir, func(paths []string) error {
+		return watcher.Watch(ctx, dir, func(events []watcher.Event) error {
 			select {
-			case event <- paths:
+			case eventCh <- events:
 			case <-ctx.Done():
 			}
 			return nil
@@ -125,10 +127,11 @@ func TestCreate(t *testing.T) {
 	time.Sleep(waitForEvents)
 	err := os.WriteFile(filepath.Join(dir, "a.txt"), []byte("b"), 0644)
 	is.NoErr(err)
-	paths, err := getEvent(event)
+	events, err := getEvent(eventCh)
 	is.NoErr(err)
-	is.Equal(len(paths), 1)
-	is.Equal(paths[0], filepath.Join(dir, "a.txt"))
+	is.Equal(len(events), 1)
+	is.Equal(events[0].Path, filepath.Join(dir, "a.txt"))
+	is.Equal(events[0].Op, watcher.OpCreate)
 	cancel()
 	is.NoErr(eg.Wait())
 }
@@ -137,14 +140,14 @@ func TestCreateRecursive(t *testing.T) {
 	is := is.New(t)
 	dir := t.TempDir()
 	ctx := context.Background()
-	event := make(chan []string)
+	eventCh := make(chan []watcher.Event)
 	ctx, cancel := context.WithCancel(context.Background())
 	defer cancel()
 	eg := new(errgroup.Group)
 	eg.Go(func() error {
-		return watcher.Watch(ctx, dir, func(paths []string) error {
+		return watcher.Watch(ctx, dir, func(events []watcher.Event) error {
 			select {
-			case event <- paths:
+			case eventCh <- events:
 			case <-ctx.Done():
 			}
 			return nil
@@ -155,11 +158,13 @@ func TestCreateRecursive(t *testing.T) {
 	is.NoErr(err)
 	err = os.WriteFile(filepath.Join(dir, "b", "a.txt"), []byte("b"), 0644)
 	is.NoErr(err)
-	paths, err := getEvent(event)
+	events, err := getEvent(eventCh)
 	is.NoErr(err)
-	is.Equal(len(paths), 2)
-	is.Equal(paths[0], filepath.Join(dir, "b"))
-	is.Equal(paths[1], filepath.Join(dir, "b/a.txt"))
+	is.Equal(len(events), 2)
+	is.Equal(events[0].Path, filepath.Join(dir, "b"))
+	is.Equal(events[0].Op, watcher.OpCreate)
+	is.Equal(events[1].Path, filepath.Join(dir, "b/a.txt"))
+	is.Equal(events[0].Op, watcher.OpCreate)
 	cancel()
 	is.NoErr(eg.Wait())
 }
@@ -168,14 +173,14 @@ func TestWithScaffold(t *testing.T) {
 	is := is.New(t)
 	dir := t.TempDir()
 	ctx := context.Background()
-	event := make(chan []string)
+	eventCh := make(chan []watcher.Event)
 	ctx, cancel := context.WithCancel(context.Background())
 	defer cancel()
 	eg := new(errgroup.Group)
 	eg.Go(func() error {
-		return watcher.Watch(ctx, dir, func(paths []string) error {
+		return watcher.Watch(ctx, dir, func(events []watcher.Event) error {
 			select {
-			case event <- paths:
+			case eventCh <- events:
 			case <-ctx.Done():
 			}
 			return nil
@@ -188,17 +193,22 @@ func TestWithScaffold(t *testing.T) {
 		"view/show.svelte":         `<h1>show</h1>`,
 	})
 	is.NoErr(err)
-	paths, err := getEvent(event)
+	events, err := getEvent(eventCh)
 	is.NoErr(err)
-	is.Equal(len(paths), 5)
-	is.Equal(paths[0], filepath.Join(dir, "controller"))
-	is.Equal(paths[1], filepath.Join(dir, "controller/controller.go"))
-	is.Equal(paths[2], filepath.Join(dir, "view"))
-	is.Equal(paths[3], filepath.Join(dir, "view/index.svelte"))
-	is.Equal(paths[4], filepath.Join(dir, "view/show.svelte"))
+	is.Equal(len(events), 5)
+	is.Equal(events[0].Path, filepath.Join(dir, "controller"))
+	is.Equal(events[0].Op, watcher.OpCreate)
+	is.Equal(events[1].Path, filepath.Join(dir, "controller/controller.go"))
+	is.Equal(events[1].Op, watcher.OpCreate)
+	is.Equal(events[2].Path, filepath.Join(dir, "view"))
+	is.Equal(events[2].Op, watcher.OpCreate)
+	is.Equal(events[3].Path, filepath.Join(dir, "view/index.svelte"))
+	is.Equal(events[3].Op, watcher.OpCreate)
+	is.Equal(events[4].Path, filepath.Join(dir, "view/show.svelte"))
+	is.Equal(events[4].Op, watcher.OpCreate)
 	// Test that there's only been one event
 	select {
-	case <-event:
+	case <-eventCh:
 		t.Fatalf("unexpected extra event")
 	case <-time.Tick(waitForEvents):
 	}
@@ -210,7 +220,7 @@ func TestWithRootDotFile(t *testing.T) {
 	is := is.New(t)
 	dir := t.TempDir()
 	ctx := context.Background()
-	event := make(chan []string)
+	eventCh := make(chan []watcher.Event)
 	err := writeFiles(dir, map[string]string{
 		"controller/controller.go": `package controller`,
 		".envrc":                   `export FOO=bar`,
@@ -219,9 +229,9 @@ func TestWithRootDotFile(t *testing.T) {
 	is.NoErr(err)
 	eg := new(errgroup.Group)
 	eg.Go(func() error {
-		return watcher.Watch(ctx, dir, func(paths []string) error {
+		return watcher.Watch(ctx, dir, func(events []watcher.Event) error {
 			select {
-			case event <- paths:
+			case eventCh <- events:
 			case <-ctx.Done():
 			}
 			return nil
@@ -234,8 +244,44 @@ func TestWithRootDotFile(t *testing.T) {
 	})
 	is.NoErr(err)
 	// Get event
-	paths, err := getEvent(event)
+	events, err := getEvent(eventCh)
 	is.NoErr(err)
-	is.Equal(len(paths), 1)
-	is.Equal(paths[0], filepath.Join(dir, "controller/controller.go"))
+	is.Equal(len(events), 1)
+	is.Equal(events[0].Path, filepath.Join(dir, "controller/controller.go"))
+	is.Equal(events[0].Op, watcher.OpUpdate)
+}
+
+func TestRename(t *testing.T) {
+	is := is.New(t)
+	dir := t.TempDir()
+	err := vfs.Write(dir, vfs.Map{
+		"a.txt": []byte(`a`),
+	})
+	is.NoErr(err)
+	ctx := context.Background()
+	eventCh := make(chan []watcher.Event)
+	ctx, cancel := context.WithCancel(context.Background())
+	defer cancel()
+	eg := new(errgroup.Group)
+	eg.Go(func() error {
+		return watcher.Watch(ctx, dir, func(events []watcher.Event) error {
+			select {
+			case eventCh <- events:
+			case <-ctx.Done():
+			}
+			return nil
+		})
+	})
+	time.Sleep(waitForEvents)
+	err = os.Rename(filepath.Join(dir, "a.txt"), filepath.Join(dir, "b.txt"))
+	is.NoErr(err)
+	events, err := getEvent(eventCh)
+	is.NoErr(err)
+	is.Equal(len(events), 2)
+	is.Equal(events[0].Path, filepath.Join(dir, "b.txt"))
+	is.Equal(events[0].Op, watcher.OpCreate)
+	is.Equal(events[1].Path, filepath.Join(dir, "a.txt"))
+	is.Equal(events[1].Op, watcher.OpDelete)
+	cancel()
+	is.NoErr(eg.Wait())
 }
